/* eslint-disable class-methods-use-this */
import { BaseResource, BaseRecord, ValidationError, } from 'adminjs';
import { prepareParams, prepareProperties, convertFilter } from './utils/index.js';
export class Resource extends BaseResource {
    model;
    relations;
    static isAdapterFor(resource) {
        return !!resource.jsonSchema && !!resource.tableName;
    }
    constructor(resource) {
        super(resource);
        this.model = resource;
        this.relations = this.getRelations();
    }
    databaseName() {
        return this.model.knex?.()?.client?.config?.connection?.database ?? 'Objection';
    }
    databaseType() {
        return this.model.knex?.()?.client?.config?.client ?? 'Other';
    }
    id() {
        return this.model.tableName;
    }
    properties() {
        // AdminJS requires 1 primary key
        const idColumn = Array.isArray(this.model.idColumn)
            ? this.model.idColumn[0]
            : this.model.idColumn;
        const properties = prepareProperties({
            schema: this.model.jsonSchema.properties,
            idColumn,
            required: this.model.jsonSchema.required,
            relations: this.relations,
        });
        return properties;
    }
    property(path) {
        return this.properties().find((p) => p.path() === path) ?? null;
    }
    async count(filter) {
        const qb = this.getQueryBuilder();
        convertFilter(qb, filter);
        const result = await qb.count({ count: '*' }).first();
        return result?.count ? Number(result.count) : 0;
    }
    async find(filter, options) {
        const qb = this.getQueryBuilder(options);
        convertFilter(qb, filter);
        const results = await qb;
        if (!results?.length)
            return [];
        return results.map((r) => new BaseRecord(r, this));
    }
    async findOne(id) {
        const qb = this.getQueryBuilder();
        const result = await qb.findById(id);
        if (!result)
            return null;
        return new BaseRecord(result, this);
    }
    async findMany(ids) {
        const qb = this.getQueryBuilder();
        const results = await qb.findByIds(ids);
        if (!results?.length)
            return [];
        return results.map((r) => new BaseRecord(r, this));
    }
    async create(params) {
        try {
            const qb = this.getQueryBuilder();
            const result = await qb.insert(prepareParams(params, this.properties()));
            return result;
        }
        catch (error) {
            this.handleValidationError(error);
            return params;
        }
    }
    async update(id, params) {
        try {
            const qb = this.getQueryBuilder();
            const result = await qb
                .patchAndFetchById(id, prepareParams(params, this.properties()));
            return result;
        }
        catch (error) {
            this.handleValidationError(error);
            return params;
        }
    }
    async delete(id) {
        const qb = this.getQueryBuilder();
        await qb.deleteById(id);
    }
    getRelations() {
        const relations = this.model.getRelations();
        return Object.values(relations).reduce((memo, current) => {
            if (!current.constructor?.name || current.constructor.name !== 'BelongsToOneRelation') {
                return memo;
            }
            const foreignKey = current?.ownerProp?.cols?.[0];
            if (!foreignKey) {
                return memo;
            }
            memo[foreignKey] = current;
            return memo;
        }, {});
    }
    withPagination(queryBuilder, { limit, offset }) {
        if (limit !== null && limit && limit > 0)
            queryBuilder.limit(limit);
        if (offset !== null && offset && offset > 0)
            queryBuilder.offset(offset);
        return queryBuilder;
    }
    withSort(queryBuilder, { sortBy, direction }) {
        if (!sortBy)
            return queryBuilder;
        return queryBuilder.orderBy(sortBy, direction ?? 'asc');
    }
    getQueryBuilder({ limit = null, offset = null, sort = {} } = {}) {
        const { sortBy, direction } = sort;
        const qb = this.model.query();
        this.withPagination(qb, { limit, offset });
        this.withSort(qb, { sortBy, direction });
        return qb;
    }
    handleValidationError(error) {
        const { type, data } = error;
        if (type !== 'ModelValidation')
            throw error;
        const errors = Object.entries(data ?? {})
            .reduce((memo, [column, info]) => {
            if (!info)
                return memo;
            if (Array.isArray(info)) {
                memo[column] = {
                    message: info?.map(({ message }) => message).join('\\n'),
                };
            }
            else {
                memo[column] = { message: info };
            }
            return memo;
        }, {});
        throw new ValidationError(errors);
    }
}
//# sourceMappingURL=Resource.js.map